@@ -27,6 +27,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QPainter>
 #include <QPointer>
 #include <QPushButton>
+#include <QTextBrowser>
 #include <QToolTip>
 #include <QVBoxLayout>
 
@@ -36,11 +37,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <KSqueezedTextLabel>
 
 // Local
-#include "contextmanager.h"
 #include "imagemetainfodialog.h"
 #include "sidebar.h"
 #include <lib/archiveutils.h>
+#include <lib/contextmanager.h>
 #include <lib/eventwatcher.h>
+#include <lib/gvdebug.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/preferredimagemetainfomodel.h>
 #include <lib/document/document.h>
@@ -58,72 +60,6 @@ namespace Gwenview
 #define LOG(x) ;
 #endif
 
-/**
- * A label which fades out if its content does not fit. If the content is
- * cropped, a tooltip is shown when the mouse hovers the widget.
- */
-class FadingLabel : public QLabel
-{
-public:
-    explicit FadingLabel(QWidget* parent = 0)
-        : QLabel(parent)
-        {
-        setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-        setTextInteractionFlags(Qt::TextBrowserInteraction);
-    }
-
-    QSize minimumSizeHint() const
-    {
-        return QSize();
-    }
-
-protected:
-    void paintEvent(QPaintEvent* event)
-    {
-        QLabel::paintEvent(event);
-        if (!isCropped()) {
-            return;
-        }
-
-        QLinearGradient gradient;
-        int gradientWidth = fontMetrics().averageCharWidth() * 4;
-        if (alignment() & Qt::AlignLeft) {
-            gradient.setStart(width() - gradientWidth, 0);
-            gradient.setFinalStop(width(), 0);
-            gradient.setColorAt(0, Qt::transparent);
-            gradient.setColorAt(1, palette().color(backgroundRole()));
-        } else {
-            gradient.setStart(0, 0);
-            gradient.setFinalStop(gradientWidth, 0);
-            gradient.setColorAt(0, palette().color(backgroundRole()));
-            gradient.setColorAt(1, Qt::transparent);
-        }
-        QPainter painter(this);
-        painter.fillRect(rect(), gradient);
-    }
-
-    bool event(QEvent* event)
-    {
-        if (event->type() == QEvent::ToolTip) {
-            // Show tooltip if cropped
-            QHelpEvent* helpEvent = static_cast<QHelpEvent*>(event);
-            if (isCropped()) {
-                QToolTip::showText(helpEvent->globalPos(), text());
-            } else {
-                QToolTip::hideText();
-                event->ignore();
-            }
-            return true;
-        }
-        return QLabel::event(event);
-    }
-
-    inline bool isCropped() const
- {
-        return sizeHint().width() > width();
-    }
-};
-
 /**
  * This widget is capable of showing multiple lines of key/value pairs.
  */
@@ -131,15 +67,20 @@ class KeyValueWidget : public QWidget
 {
     struct Row
     {
-        Row()
-        : keyLabel(new FadingLabel)
-        , valueLabel(new FadingLabel)
+        Row(QWidget* parent)
+        : keyLabel(new QLabel(parent))
+        , valueLabel(new QLabel(parent))
         {
-            if (QApplication::isLeftToRight()) {
-                keyLabel->setAlignment(Qt::AlignRight);
-            } else {
-                valueLabel->setAlignment(Qt::AlignRight);
-            }
+            initLabel(keyLabel);
+            initLabel(valueLabel);
+
+            QPalette pal = keyLabel->palette();
+            QColor color = pal.color(QPalette::WindowText);
+            color.setAlphaF(0.65);
+            pal.setColor(QPalette::WindowText, color);
+            keyLabel->setPalette(pal);
+
+            valueLabel->setContentsMargins(6, 0, 0, 6);
         }
 
         ~Row()
@@ -148,40 +89,55 @@ class KeyValueWidget : public QWidget
             delete valueLabel;
         }
 
-        FadingLabel* keyLabel;
-        FadingLabel* valueLabel;
+        int setLabelGeometries(int rowY, int labelWidth)
+        {
+            int labelHeight = keyLabel->heightForWidth(labelWidth);
+            keyLabel->setGeometry(0, rowY, labelWidth, labelHeight);
+            rowY += labelHeight;
+            labelHeight = valueLabel->heightForWidth(labelWidth);
+            valueLabel->setGeometry(0, rowY, labelWidth, labelHeight);
+            rowY += labelHeight;
+            return rowY;
+        }
+
+        int heightForWidth(int width) const
+        {
+            return keyLabel->heightForWidth(width) + valueLabel->heightForWidth(width);
+        }
+
+        static void initLabel(QLabel* label)
+        {
+            label->setWordWrap(true);
+            label->show();
+            label->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::LinksAccessibleByMouse);
+        }
+
+        QLabel* keyLabel;
+        QLabel* valueLabel;
     };
 public:
     KeyValueWidget(QWidget* parent)
     : QWidget(parent)
-    , mLayout(new QGridLayout(this))
     {
-        mLayout->setMargin(0);
-        mLayout->setVerticalSpacing(0);
-        mLayout->setHorizontalSpacing(4);
-        setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
+        QSizePolicy policy(QSizePolicy::Preferred, QSizePolicy::Fixed);
+        policy.setHeightForWidth(true);
+        setSizePolicy(policy);
     }
 
     QSize sizeHint() const
     {
-        int height = fontMetrics().height() * mRows.count();
-        return QSize(150, height);
+        int width = 150;
+        int height = heightForWidth(width);
+        return QSize(width, height);
     }
 
-    void addRow(const QString& key, const QString& value)
+    int heightForWidth(int w) const
     {
-        Row* row = new Row;
-        row->keyLabel->setText(i18nc(
-                                   "@item:intext %1 is a key, we append a colon to it. A value is displayed after",
-                                   "%1:", key));
-        row->valueLabel->setText(value);
-        mRows.append(row);
-
-        int rowCount = mLayout->rowCount();
-        mLayout->addWidget(row->keyLabel, rowCount, 0);
-        mLayout->addWidget(row->valueLabel, rowCount, 1);
-        updateKeyColumnWidth();
-        updateGeometry();
+        int height = 0;
+        Q_FOREACH(Row* row, mRows) {
+             height += row->heightForWidth(w);
+        }
+        return height;
     }
 
     void clear()
@@ -191,32 +147,44 @@ public:
         updateGeometry();
     }
 
-protected:
-    void resizeEvent(QResizeEvent* event)
+    void addRow(const QString& key, const QString& value)
     {
-        QWidget::resizeEvent(event);
-        updateKeyColumnWidth();
+        Row* row = new Row(this);
+        row->keyLabel->setText(i18nc(
+           "@item:intext %1 is a key, we append a colon to it. A value is displayed after",
+           "%1:", key));
+        row->valueLabel->setText(value);
+        mRows << row;
         updateGeometry();
     }
 
-private:
-    QList<Row*> mRows;
-    QGridLayout* mLayout;
-
-    void updateKeyColumnWidth()
+    void layoutRows()
     {
-        const int maxWidth = width() / 2;
-        int keyWidth = 0;
-        Q_FOREACH(Row * row, mRows) {
-            int wantedWidth = row->keyLabel->sizeHint().width();
-            if (wantedWidth > keyWidth) {
-                keyWidth = qMin(wantedWidth, maxWidth);
-            }
-        }
-        Q_FOREACH(Row * row, mRows) {
-            row->keyLabel->setFixedWidth(keyWidth);
+        // Layout labels manually: I tried to use a QVBoxLayout but for some
+        // reason when using software animations the widget blinks when going
+        // from one image to another
+        int rowY = 0;
+        const int labelWidth = width();
+        Q_FOREACH(Row* row, mRows) {
+            rowY = row->setLabelGeometries(rowY, labelWidth);
         }
     }
+
+protected:
+    void showEvent(QShowEvent* event)
+    {
+        QWidget::showEvent(event);
+        layoutRows();
+    }
+
+    void resizeEvent(QResizeEvent* event)
+    {
+        QWidget::resizeEvent(event);
+        layoutRows();
+    }
+
+private:
+    QVector<Row*> mRows;
 };
 
 struct InfoContextManagerItemPrivate
@@ -371,20 +339,17 @@ void InfoContextManagerItem::updateOneFileInfo()
         return;
     }
 
-    QStringList list;
-    d->mKeyValueWidget->clear();
     ImageMetaInfoModel* metaInfoModel = d->mDocument->metaInfo();
+    d->mKeyValueWidget->clear();
     Q_FOREACH(const QString & key, GwenviewConfig::preferredMetaInfoKeyList()) {
         QString label;
         QString value;
         metaInfoModel->getInfoForKey(key, &label, &value);
-
         if (!label.isEmpty() && !value.isEmpty()) {
             d->mKeyValueWidget->addRow(label, value);
         }
     }
-
-    d->mKeyValueWidget->show();
+    d->mKeyValueWidget->layoutRows();
 }
 
 void InfoContextManagerItem::showMetaInfoDialog()
