@@ -70,6 +70,8 @@ PreferredImageMetaInfoModel::PreferredImageMetaInfoModel(ImageMetaInfoModel* mod
 {
     d->mModel = model;
     setSourceModel(model);
+    sort(0);
+    setDynamicSortFilter(true);
     d->mPreferredMetaInfoKeyList = list;
 }
 
@@ -127,4 +129,15 @@ bool PreferredImageMetaInfoModel::setData(const QModelIndex& index, const QVaria
     return true;
 }
 
+bool PreferredImageMetaInfoModel::lessThan(const QModelIndex& left, const QModelIndex& right) const
+{
+    if (!left.parent().isValid()) {
+        // Keep root entries in insertion order
+        return left.row() < right.row();
+    } else {
+        // Sort leaf entries alphabetically
+        return QSortFilterProxyModel::lessThan(left, right);
+    }
+}
+
 } // namespace
